#include "ProjectileMotion.h"
#include "VectorMath.h"
#include <iostream>
#include <iomanip>
#include <sstream>
#include <cmath>
#include <thread>
#include <utility>

constexpr double target_range = 100;//meters
constexpr double target_direction = 0;//degrees (measures from North, going clockwise)
constexpr double wind_speed = 0;//m/s
constexpr double wind_direction = 0;//degrees (measures from North, going clockwise)

using namespace std;

//calculates the distance between two points on a 2-D plane
double find_distance(pair<double, double> location, pair<double, double> target)
{
	return sqrt(pow(location.first - target.first, 2) + pow(location.second - target.second, 2));
}

//turns a polar coordinate into a rectangular coordinate
pair<double, double> polar_to_rect(double range, double direction)
{
	return make_pair(range * cos(deg_to_rad(direction)), range * sin(deg_to_rad(direction)));
}

int main()
{
	pair<double, double> target = make_pair(target_range*cos(deg_to_rad(target_direction)), target_range *sin(deg_to_rad(target_direction)));

	double range, direction;
	double v_plus_range, v_plus_direction;
	double v_minus_range, v_minus_direction;
	double theta_plus_range, theta_plus_direction;
	double theta_minus_range, theta_minus_direction;

	double launch_speed = pow(target_range * 9.81, 0.5);
	double launch_angle = 35;
	double launch_direction = target_direction;

	do
	{
		//simultaneously calculate 5 slightly-different projectile motions
		thread t1(motion_calculation, launch_angle, launch_speed, launch_direction, wind_speed, wind_direction, ref(range), ref(direction));
		thread t2(motion_calculation, launch_angle, launch_speed + 0.05, launch_direction, wind_speed, wind_direction, ref(v_plus_range), ref(v_plus_direction));
		thread t3(motion_calculation, launch_angle, launch_speed - 0.05, launch_direction, wind_speed, wind_direction, ref(v_minus_range), ref(v_minus_direction));
		thread t4(motion_calculation, launch_angle, launch_speed, launch_direction + 0.5, wind_speed, wind_direction, ref(theta_plus_range), ref(theta_plus_direction));
		thread t5(motion_calculation, launch_angle, launch_speed, launch_direction - 0.5, wind_speed, wind_direction, ref(theta_minus_range), ref(theta_minus_direction));
		if (t1.joinable()) t1.join();
		if (t2.joinable()) t2.join();
		if (t3.joinable()) t3.join();
		if (t4.joinable()) t4.join();
		if (t5.joinable()) t5.join();

		//if simulated motion is close enough to landing site
		if (find_distance(polar_to_rect(range, direction), target) <= 1) break;

		//derivatives of distance to landing site with respect to velocity and theta
		double dd_dv = (find_distance(polar_to_rect(v_plus_range, v_plus_direction), target) - find_distance(polar_to_rect(v_minus_range, v_minus_direction), target)) / 0.1;
		double dd_dtheta = (find_distance(polar_to_rect(theta_plus_range, theta_plus_direction), target) - find_distance(polar_to_rect(theta_minus_range, theta_minus_direction), target));

		//use gradient descent to adjust launch speed and launch direction
		launch_speed += dd_dv * (-0.01);
		launch_direction += dd_dtheta * (-0.1);
	} while (true);

	//prepares output information
	stringstream ss;
	ss << output_parameters()
		<< setw(22) << "Range: " << setw(10) << range << " meters" << endl
		<< setw(22) << "Landing Direction: " << setw(10) << target_direction << " degrees" << endl
		<< setw(22) << "Wind Speed: " << setw(10) << wind_speed << " m/s" << endl
		<< setw(22) << "Wind Direction: " << setw(10) << wind_direction << " degrees" << endl
		<< endl
		<< setw(22) << "Launch Speed: " << setw(10) << launch_speed << " m/s" << endl
		<< setw(22) << "Launch Angle: " << setw(10) << launch_angle << " degrees above horizontal" << endl
		<< setw(22) << "Launch Direction: " << setw(10) << launch_direction << " degrees" << endl;
	cout << ss.str();
    return 0;
}