#include "VectorMath.h"
#include <vector>
#include <string>
#include <sstream>
#include <iomanip>

constexpr double pi = 3.14159265358979324;
constexpr double drag_c = 0.47;
constexpr double air_dense = 1.225;//kg/m^3
constexpr double x_area = 0.15;//m^2
constexpr double mass = 40;//kg

using namespace std;

//angle unit conversion functions
double deg_to_rad(double degrees)
{
	return pi * degrees / 180.0;
}
double rad_to_deg(double radians)
{
	return 180.0 * radians / pi;
}

void motion_calculation(double launch_angle, double launch_speed, double launch_direction, double wind_speed, double wind_direction, double& range, double& landing_direction)
{
	//x-component is horizontal velocity along the original launch path
	//y-component is horizontal velocity perpendicular to original launch path
	//z-component is vertical velocity
	vector<double> velocity{ 0, 0, 0 };
	vector<double> position{ 0, 0, 0 };
	vector<double> wind{ 0, 0, 0 };
	vector<double> acceleration;

	velocity[0] = launch_speed * cos(deg_to_rad(launch_angle)) * cos(deg_to_rad(launch_direction));//sets x-velocity m/s
	velocity[1] = launch_speed * cos(deg_to_rad(launch_angle)) * sin(deg_to_rad(launch_direction));//sets y-velocity m/s
	velocity[2] = launch_speed * sin(deg_to_rad(launch_angle));//sets z-velocity m/s
	double speed = launch_speed;//sets velocity

	wind[0] = wind_speed * cos(deg_to_rad(wind_direction));//sets x-velocity m/s
	wind[1] = wind_speed * sin(deg_to_rad(wind_direction));//sets y-velocity m/s

	double accel_drag; //initialization: how much speed lost due to drag

	double dt = 0.001; //time interval between calculations

	//simulates projectile motion in air
	do
	{
		//takes care of position change using velocity
		position = vector_add(position, scalar_multiplication(dt, velocity));

		//defines acceleration due to drag and gravity
		vector<double> wind_relative_velocity = vector_subtract(wind, velocity);
		accel_drag = 0.5 * drag_c * air_dense * x_area * pow(magnitude(wind_relative_velocity), 2) / mass;
		acceleration = scalar_multiplication(accel_drag, normalize(wind_relative_velocity));
		acceleration[2] -= 9.8;

		//changes velocity due to acceleration
		velocity = vector_add(velocity, scalar_multiplication(dt, acceleration));
	} while (position[2] > 0);//repeat while hasn't hit ground yet

	//calculates landing distance and direction
	//uses references to "return" multiple values
	position.pop_back();
	range = magnitude(position);
	landing_direction = rad_to_deg(atan2(position[1], position[0]));
}

string output_parameters()
{
	stringstream ss;
	ss << setw(22) << "Mass: " << setw(10) << mass << " kilograms" << endl
		<< setw(22) << "Drag Coefficient: " << setw(10) << drag_c << endl
		<< setw(22) << "Air Density: " << setw(10) << air_dense << " kg/m^3" << endl
		<< setw(22) << "Cross-sectional Area: " << setw(10) << x_area << " m^2" << endl
		<< endl;
	return ss.str();
}